#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>
#include <flib.h>
#include "vl.h"

#define LUNG 30
#define CENTRE_DISTANCE 40
#define VX 4
#define VY 4
#define MINX 16
#define MINY 16

void mouseFunc(int, int, int, int, void*);

int x1, x2, y1, y2, waitSelected = 0;

int main(void)
{
	image i;
	float vmax, v;
	int x, y, xmax, ymax;
	U32 h1[256], h2[256];
	init_licence("LT307EB4FAC1");
	
	vmode(vmLive);
	
	ScrSetPhysPage((int)ScrGetPhysPage + 1);
	ScrSetLogPage((int)ScrGetPhysPage);
	ImageAssign(&i, ScrByteAddr(0 , 0), DX, DY, ScrGetPitch);
	
	if (vlNumberedWindow(352) != VL_NO_ERROR)
		return 1;
	
	vlSetMouseCallback(352, mouseFunc, NULL);
	
	printf("Select the object to track\n");
	
	for (; vlWaitKey(352, 10) != 'q' && !waitSelected;)
	{
		tpict();
		
		if (vlShowImage(352) != VL_NO_ERROR)
			return 1;
		
		searchForEvents();
		if(kbhit() && rs232rcv() == 'q')
			break;
	}
	
	//printf("OK\n");
	drawRect(&i, x1, y1, x2, y2, 0xFF);
	if (vlShowImage(352) != VL_NO_ERROR)
		return 1;
	printf("%d %d %d %d\n", x1, x2, y1, y2);
	calcHist(&i, h1, x1, y1, x2, y2);
	//printf("aa\n");
	
	for (; 1;)
	{
		//printf("bb\n");
		tpict();
		//printf("GIRO\n");
		
		vmax = -9999999;
		for(y = y1 - CENTRE_DISTANCE; y < y1 + CENTRE_DISTANCE; y += 2)
		{
			for(x = x1 + y % 4 - CENTRE_DISTANCE; x < x1 + CENTRE_DISTANCE; x += 4)
			{
				if ((x >= DX) || (y >= DY) || (x < 0) || (y < 0))
				{
					continue;
				}
				
				calcHist(&i, h2, x, y, x + x2 - x1, y + y2 - y1);
				
				v = compHist(h1, h2);
				//printf("%f\n", v);
				
				if (v > vmax)
				{
					vmax = v;
					xmax = x;
					ymax = y;
				}
			}
		}
		
		x2 = xmax + x2 - x1;
		y2 = ymax + y2 - y1;
		x1 = xmax;
		y1 = ymax;
		
		//Resize hist rectangle
		calcHist(&i, h2, x1 + VX / 2, y1, x2, y2);
		v = compHist(h1, h2);
		if (v < vmax)
		{
			vmax = v;
			if (x2 - x1 > MINX)
				x1 += VX / 2;
		}
		calcHist(&i, h2, x1 - VX / 2, y1, x2, y2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			x1 -= VX / 2;
		}
		calcHist(&i, h2, x1, y1, x2 + VX / 2, y2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			x2 += VX / 2;
		}
		calcHist(&i, h2, x1, y1, x2 - VX / 2, y2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			if (x2 - x1 > MINX)
				x2 -= VX / 2;
		}
		calcHist(&i, h2, x1, y1 - VY / 2, x2, y2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			y1 -= VY / 2;
		}
		calcHist(&i, h2, x1, y1 + VY / 2, x2, y2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			if (y2 - y1 > MINY)
				y1 += VY / 2;
		}
		calcHist(&i, h2, x1, y1, x2, y2 + VY / 2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			y2 += VY / 2;
		}
		calcHist(&i, h2, x1, y1, x2, y2 - VY / 2);
		v = compHist(h1, h2);
		if (v > vmax)
		{
			vmax = v;
			if (y2 - y1 > MINY)
				y2 -= VY / 2;
		}
		
		printf("x1: %d, y1: %d, x2: %d, y2: %d\n", x1, y1, x2, y2);
		
		drawRect(&i, x1, y1, x2, y2, 0xFF);
		
		if (vlShowImage(352) != VL_NO_ERROR)
			return 1;
		searchForEvents();
		if(kbhit() && rs232rcv() == 'q')
			break;
	}
	
	if (vlDestroyWindows(352) != VL_NO_ERROR)
		return 1;
	
	vmode(vmStill);
	
	return 0;
}

void mouseFunc(int event, int x, int y, int flags, void* param)
{
	if (event == VL_EVENT_LBUTTONUP)
	{
		x1 = x - LUNG / 2;
		y1 = y - LUNG / 2;
		x2 = x + LUNG / 2;
		y2 = y + LUNG / 2;
		waitSelected = 1;
	}
}
